#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import logging

from viewer.__data__ import ERROR_BOOTLOADER_ERROR
from viewer.__data__ import ERROR_CMDLINE_READ
from viewer.__data__ import ERROR_CREATE_ARRAY
from viewer.__data__ import ERROR_CREATE_LVM
from viewer.__data__ import ERROR_CREATE_PARTITIONS
from viewer.__data__ import ERROR_DELETE_PARTITIONS
from viewer.__data__ import ERROR_DISKS_SIZE
from viewer.__data__ import ERROR_DISK_LABEL_ERROR
from viewer.__data__ import ERROR_DISK_SCAN
from viewer.__data__ import ERROR_DOWNLOAD_HTTP
from viewer.__data__ import ERROR_DOWNLOAD_NFS
from viewer.__data__ import ERROR_DOWNLOAD_REPODATA_FAIL
from viewer.__data__ import ERROR_DOWNLOAD_TFTP
from viewer.__data__ import ERROR_ENTITLEMENT
from viewer.__data__ import ERROR_ENTITLE_FW
from viewer.__data__ import ERROR_ENTITLE_HYPER
from viewer.__data__ import ERROR_ENTITLE_MACHINE
from viewer.__data__ import ERROR_ERROR
from viewer.__data__ import ERROR_EXEC_POSTSCRIPT
from viewer.__data__ import ERROR_EXEC_PRESCRIPT
from viewer.__data__ import ERROR_HWCLOCK_ERROR
from viewer.__data__ import ERROR_INEXISTENT_LV
from viewer.__data__ import ERROR_INVALID_NIC
from viewer.__data__ import ERROR_INVALID_PV
from viewer.__data__ import ERROR_INVALID_REPO
from viewer.__data__ import ERROR_INVALID_REPO_CONTENT
from viewer.__data__ import ERROR_INVALID_VG
from viewer.__data__ import ERROR_KS_CANNOT_PRESERVE
from viewer.__data__ import ERROR_KS_INVALID_PV_NAME
from viewer.__data__ import ERROR_KS_MISSING_LVM
from viewer.__data__ import ERROR_KS_MULTIPLE_VGS
from viewer.__data__ import ERROR_KS_NO_FORMAT
from viewer.__data__ import ERROR_KS_PRESERV_VOLGROUP_PART
from viewer.__data__ import ERROR_KS_REPEATED_PVS
from viewer.__data__ import ERROR_KS_SIZE_NO_FORMAT
from viewer.__data__ import ERROR_KS_UNKNOWN_PV
from viewer.__data__ import ERROR_KS_UNKNOWN_VG
from viewer.__data__ import ERROR_KS_VOLGROUP_PART
from viewer.__data__ import ERROR_LVS_SIZE
from viewer.__data__ import ERROR_LV_SIZE
from viewer.__data__ import ERROR_MEDIA_DIR_UMOUNT_ERROR
from viewer.__data__ import ERROR_MULTIPATH_ERROR
from viewer.__data__ import ERROR_NO_DISK
from viewer.__data__ import ERROR_NO_NIC
from viewer.__data__ import ERROR_NO_REPO
from viewer.__data__ import ERROR_PARSER_FILE
from viewer.__data__ import ERROR_PKG_INSTALL_ERROR
from viewer.__data__ import ERROR_PKG_REPO_MOUNT_ERROR
from viewer.__data__ import ERROR_POST_MODULES
from viewer.__data__ import ERROR_PRE_MODULES
from viewer.__data__ import ERROR_REPO_URL
from viewer.__data__ import ERROR_ROOT_PASSWD_CHANGE_ERROR
from viewer.__data__ import ERROR_ROOT_PWD_MANUAL
from viewer.__data__ import ERROR_SELINUX_FS_LABEL_ERROR
from viewer.__data__ import ERROR_SELINUX_RESTORE_ERROR
from viewer.__data__ import ERROR_TIMEZONE_LIST
from viewer.__data__ import ERROR_UNEXPECTED
from viewer.__data__ import ERROR_UNEXPECTED_AUTOINSTALL
from viewer.__data__ import ERROR_UNEXPECTED_DISKSEL
from viewer.__data__ import ERROR_UNEXPECTED_PACKAGES
from viewer.__data__ import ERROR_UNEXPECTED_PARTITIONER
from viewer.__data__ import ERROR_UNEXPECTED_POSTINSTALL
from viewer.__data__ import ERROR_UTC_CONF
from viewer.__data__ import ERROR_YUM_CACHE_CREATE_ERROR
from viewer.__data__ import ERROR_YUM_CACHE_UMOUNT_ERROR
from viewer.__data__ import ERROR_YUM_INSTANCE_ERROR


#
# CONSTANTS
#
ERROR_PREFIX = "IBMKVM"


#
# CODE
#
class PKVMError(Exception):
    """
    USAGE:
    =====
        * Raise:
          -----
            from controller.pkvmerror import PKVMError

            try:
                <code...>
            except Exception as e:
                # raise with specific error code
                raise PKVMError("002")

        * Handle:
          ------

            + General use case:
              ----------------
                from controller.pkvmerror import PKVMError

                try:
                    <code...>
                except PKVMError as e:
                    raise
                except Exception as e:
                    # raise with 'unexpected error' code related to the
                    # current installer component
                    raise PKVMError("004")

            + Entry points (only controller's __init__()):
              ----------------------------------------------
                from controller.pkvmerror import PKVMError
                from logger import createFailureLog

                except PKVMError as e:
                    createFailureLog()
                    viewInput = {"errorMsg": e.getErrorMessage()}
                    self.__viewer.runView(ScreenIds.GENERAL_ERROR, viewInput)
                    self.rebootSystem(True)
                except Exception as e:
                    # general 'unexpected error' code
                    pkvmError = PKVMError("000")
                    createFailureLog()
                    viewInput = {"errorMsg": pkvmError.getErrorMessage()}
                    self.__viewer.runView(ScreenIds.GENERAL_ERROR, viewInput)
                    self.rebootSystem(True)
    """

    def __init__(self, code, args={}):
        self.__code = code
        self.__args = args
        self.__logger = logging.getLogger(__name__)
        self.__logger.exception(self.__getErrorCode())

        self.__errorMessages = {
            "000": ERROR_UNEXPECTED.localize(),
            "001": ERROR_DISKS_SIZE.localize(),
            "002": ERROR_NO_DISK.localize(),
            "003": ERROR_ENTITLEMENT.localize(),
            "004": ERROR_UNEXPECTED_PARTITIONER.localize(),
            "005": ERROR_UNEXPECTED_PACKAGES.localize(),
            "006": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "007": ERROR_UNEXPECTED_AUTOINSTALL.localize(),
            "008": ERROR_UNEXPECTED_DISKSEL.localize(),
            "009": ERROR_DELETE_PARTITIONS.localize(),
            "010": ERROR_CREATE_PARTITIONS.localize(),
            "011": ERROR_CREATE_LVM.localize(),
            "012": ERROR_INVALID_PV.localize(),
            "013": ERROR_INVALID_VG.localize(),
            "014": ERROR_CREATE_ARRAY.localize(),
            "015": ERROR_DISK_SCAN.localize(),
            "016": ERROR_MULTIPATH_ERROR.localize(),
            "017": ERROR_BOOTLOADER_ERROR.localize(),
            "018": ERROR_DOWNLOAD_HTTP.localize(),
            "019": ERROR_DOWNLOAD_TFTP.localize(),
            "020": ERROR_DOWNLOAD_NFS.localize(),
            "021": ERROR_PARSER_FILE.localize(),
            "022": ERROR_PRE_MODULES.localize(),
            "023": ERROR_EXEC_PRESCRIPT.localize(),
            "024": ERROR_POST_MODULES.localize(),
            "025": ERROR_ROOT_PWD_MANUAL.localize(),
            "026": ERROR_EXEC_POSTSCRIPT.localize(),
            "027": ERROR_INVALID_NIC.localize(),
            "028": ERROR_NO_NIC.localize(),
            "029": ERROR_TIMEZONE_LIST.localize(),
            "030": ERROR_UTC_CONF.localize(),
            "031": ERROR_NO_REPO.localize(),
            "032": ERROR_INVALID_REPO.localize(),
            "033": ERROR_INVALID_REPO_CONTENT.localize(),
            "034": ERROR_YUM_INSTANCE_ERROR.localize(),
            "035": ERROR_YUM_CACHE_CREATE_ERROR.localize(),
            "036": ERROR_HWCLOCK_ERROR.localize(),
            "037": ERROR_PKG_INSTALL_ERROR.localize(),
            "038": ERROR_YUM_CACHE_UMOUNT_ERROR.localize(),
            "039": ERROR_MEDIA_DIR_UMOUNT_ERROR.localize(),
            "040": ERROR_ROOT_PASSWD_CHANGE_ERROR.localize(),
            "041": ERROR_SELINUX_FS_LABEL_ERROR.localize(),
            "042": ERROR_SELINUX_RESTORE_ERROR.localize(),
            "043": ERROR_CREATE_PARTITIONS.localize(),
            "044": ERROR_CREATE_PARTITIONS.localize(),
            "045": ERROR_CREATE_PARTITIONS.localize(),
            "047": ERROR_DISK_LABEL_ERROR.localize(),
            "048": ERROR_PKG_REPO_MOUNT_ERROR.localize(),
            "049": ERROR_ENTITLE_FW.localize(),
            "050": ERROR_ENTITLE_MACHINE.localize(),
            "051": ERROR_ENTITLE_HYPER.localize(),
            "052": ERROR_DOWNLOAD_REPODATA_FAIL.localize(),
            "053": ERROR_KS_MULTIPLE_VGS.localize(),
            "054": ERROR_KS_REPEATED_PVS.localize(),
            "055": ERROR_KS_INVALID_PV_NAME.localize(),
            "056": ERROR_KS_UNKNOWN_PV.localize(),
            "057": ERROR_KS_UNKNOWN_VG.localize(),
            "058": ERROR_LVS_SIZE.localize(),
            "059": ERROR_LV_SIZE.localize(),
            "060": ERROR_KS_SIZE_NO_FORMAT.localize(),
            "061": ERROR_KS_NO_FORMAT.localize(),
            "062": ERROR_KS_CANNOT_PRESERVE.localize(),
            "063": ERROR_INEXISTENT_LV.localize(),
            "064": ERROR_KS_VOLGROUP_PART.localize(),
            "065": ERROR_KS_PRESERV_VOLGROUP_PART.localize(),
            "066": ERROR_KS_MISSING_LVM.localize(),
            "067": ERROR_CMDLINE_READ.localize(),
            "068": ERROR_NO_REPO.localize(),
            "070": ERROR_CREATE_PARTITIONS.localize(),
            "071": ERROR_NO_DISK.localize(),
            "072": ERROR_NO_DISK.localize(),
            "073": ERROR_INVALID_VG.localize(),
            "074": ERROR_MULTIPATH_ERROR.localize(),
            "075": ERROR_MULTIPATH_ERROR.localize(),
            "076": ERROR_MULTIPATH_ERROR.localize(),
            "077": ERROR_MULTIPATH_ERROR.localize(),
            "078": ERROR_BOOTLOADER_ERROR.localize(),
            "079": ERROR_PARSER_FILE.localize(),
            "080": ERROR_PARSER_FILE.localize(),
            "081": ERROR_PARSER_FILE.localize(),
            "082": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "083": ERROR_PRE_MODULES.localize(),
            "084": ERROR_PRE_MODULES.localize(),
            "085": ERROR_PRE_MODULES.localize(),
            "086": ERROR_PRE_MODULES.localize(),
            "087": ERROR_PRE_MODULES.localize(),
            "088": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "089": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "090": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "091": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "092": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "093": ERROR_UNEXPECTED_POSTINSTALL.localize(),
            "094": ERROR_INVALID_REPO.localize(),
            "095": ERROR_CREATE_PARTITIONS.localize(),
            "096": ERROR_BOOTLOADER_ERROR.localize(),
            "097": ERROR_UNEXPECTED_PARTITIONER.localize(),
            "098": ERROR_UNEXPECTED_PARTITIONER.localize(),
            "099": ERROR_DELETE_PARTITIONS.localize(),
            "100": ERROR_CREATE_PARTITIONS.localize(),
            "101": ERROR_CREATE_LVM.localize(),
            "102": ERROR_CREATE_LVM.localize(),
            "103": ERROR_CREATE_ARRAY.localize(),
            "104": ERROR_UNEXPECTED_PARTITIONER.localize(),
            "105": ERROR_CREATE_LVM.localize(),
        }


    def __getErrorCode(self):
        """
        Return error code

        Returns:
            str: error code
        """
        return ERROR_PREFIX + self.__code

    def getErrorMessage(self):
        """
        Return error message

        Returns:
            str: error message
        """

        return (self.__getErrorCode() + ": " +
                self.__errorMessages[self.__code] % self.__args)
