#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from yum.constants import TS_INSTALL_STATES, TS_ERASE
from yum.rpmtrans import RPMBaseCallback


#
# CONSTANTS AND DEFINITIONS
#
FOLLOW_PROGRESS = { "operation": "",
                    "percent": "",
                    "package": ""
                   }


#
# CODE
#
class InstallationCallback(RPMBaseCallback):
    """
    Handles the progress of the installation
    """

    def __init__(self, callback, num_pkgs):
        """
        Constructor

        Args:
            callback (method): user callback
        """
        # call parent constructor
        RPMBaseCallback.__init__(self)

        # set user callback
        self.__userCallback = callback
        self.__num_pkgs = num_pkgs

        # create start lock
        self.__startLock = {}

    def errorlog(self, msg):
        """
        Dummy method. Just ignores any error messages sent from the RPM
        installation process.

        Args:
            msg (basestring): RPM installation error messages
        """
        pass

    def event(self, package, action, bytesProcessed, bytesTotal, currentProcess, totalProcess):
        """
        Handles installation progress

        Args:
            package (basestring): A yum package object or simple string of a package name
            action (int): A yum.constant transaction set state or in the obscure
                rpm repackage case it could be the string 'repackaging'
            bytesProcessed (int): Current number of bytes processed in the transaction
                element being processed
            bytesTotal (int): Total number of bytes in the transaction element being
                processed
            currentProcess (int): number of processes completed in whole transaction
            totalProcess (int): total number of processes in the transaction.
        """
        # action is install: get package name and save state
        if action in TS_INSTALL_STATES:
            packageSplited  = package._remote_url().split('/')
            packageName = packageSplited[ len(packageSplited) - 1]
            state = 'autoinstall'

        # state is uninstall: get package name and save state
        elif action == TS_ERASE:
            packageName = package
            state = 'uninstall'

        # other action: ignore
        else:
            return

        # calculate percentage
        percentage = (float(bytesProcessed) / float(bytesTotal) ) * 100

        # create message
        message = FOLLOW_PROGRESS

        # lock not exists: operation is starting
        if self.__startLock.get(packageName) == None:
            message['operation'] = '%s-start' % state
            self.__startLock[packageName] = True

        # percentage is 100: operation finish
        elif percentage == 100:
            message['operation'] = '%s-finish' % state

        # operation is running
        else:
            message['operation'] = '%s' % state

        # fill message dict
        message['percent'] = percentage
        message['package'] = packageName
        message['total_pkgs'] = self.__num_pkgs

        # report user callback
        self.__userCallback(message)

    def filelog(self, package, action):
        """
        Dummy method. Just ignores any log messages sent from the RPM
        installation process.

        Args:
            package (basestring): package name
            action (basestring): RPM installation log message
        """
        pass

    def scriptout(self, package, msgs):
        """
        Dummy method. Just ignores any scriptlet output messages sent from the
        RPM installation process.

        Args:
            package (basestring): package name
            msgs (basestring): RPM scriptlet output messages
        """
        pass
