#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from urlgrabber.progress import BaseMeter


#
# CONSTANTS
#


#
# CODE
#
class MetadataDownloadCallback(BaseMeter):
    """
    Follow the metadata download progress
    """

    def __init__(self, parseCallback, userCallback):
        """
        Constructor

        Args:
            parseCallback (object): repository parser callback
            userCallback (dict): user callback
        """
        # call parent constructor
        BaseMeter.__init__(self)

        # set update frequency as 10 Hz
        self.update_period = 0.1

        # set user callback
        self.__userCallback = userCallback

        # set parser callback
        self.__parseCallback = parseCallback

    def _do_end(self, amount_read, now=None):
        """
        Download finish callback

        IMPORTANT: the implementation of this method is part of the Yum API for
        callbacks. Do not change the name of this method.

        Args:
            amount_read (int): amount downloaded
            now (float): current time
        """
        # call user-defined callback
        self.__userCallback({
            'operation': 'download-finish',
            'package': self.filename,
            'url': self.url,
        })

        # set filename in parse callback
        self.__parseCallback.setFileName(self.filename)

    def _do_start(self, now):
        """
        Download start callback

        IMPORTANT: the implementation of this method is part of the Yum API for
        callbacks. Do not change the name of this method.

        Args:
            now (float): current time
        """
        self.__userCallback({
            'operation': 'download-start',
            'package': self.filename,
            'url': self.url,
        })

    def _do_update(self, amount_read, now = None):
        """
        Download progress callback

        IMPORTANT: the implementation of this method is part of the Yum API for
        callbacks. Do not change the name of this method.

        Args:
            amount_read (int): amount downloaded
            now (float): current time
        """
        # try to calculate download progress percentage
        try:
            percent = 100 * float(amount_read) / self.size

        # division by zero: set percent as zero
        except ZeroDivisionError:
            percent = 0

        # call user-defined callback
        self.__userCallback({
            'operation': 'download',
            'package': self.filename,
            'url': self.url,
            'percent': percent,
        })
