#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


def readVersion():
    """
    Read PowerKVM version from disk
    Supposes that PowerKVM release rpm package was installed

    Returns:
        str: PowerKVM version string
    """
    _filename = '/etc/os-release'
    d = dict()

    with open(_filename) as file_handler:
        for line in file_handler:
            sline = line.rstrip('\n').strip()
            if sline.count('=') > 0:
                (varname,value) = sline.split('=')
                d[varname] = value.strip('\"')

    stream = d.get('VERSION_ID')
    if stream is None:
        raise Exception("Invalid PowerKVM version information in /etc/os-release")
    service_pack = d.get('SERVICE_ID')
    if service_pack is None:
        raise Exception("Invalid PowerKVM service version information in /etc/os-release")
    full_build = d.get('BUILD_ID')
    if full_build is None:
        raise Exception("Invalid PowerKVM build version information in /etc/os-release")
    if full_build.count('.') == 1:
        (build,respin) = full_build.split('.')
    else:
        build = full_build
        respin = 0

    return "%s.%s.%s.%s" % (stream, service_pack, build, respin)


STR_VERSION = readVersion()

MIN_PASSWORD_LENGTH = 6

# Project root path.
RUN_DIR = '/opt/ibm/powerkvm'

# Root filesystem image
ROOTFS_IMG = '/mnt/rootfs.img'

# New root filesystem image
NEW_ROOTFS_IMG = 'newrootfs.img'

# Error file: indicates the installation or update process failed
OPERATION_FAILED = '/opt/ibm/powerkvm/control/.install-failed'

# Information file: contains information about the installation or update process
OPERATION_INFO = '/opt/ibm/powerkvm/control/.install-info'

# Progress file: indicates the operation progress
OPERATION_PROGRESS = '/opt/ibm/powerkvm/control/.install-progress'

# Success file: indicates the operation successfully completes
OPERATION_COMPLETED = '/opt/ibm/powerkvm/control/.install-success'

# PowerKVM partition layout
PARTITION_LAYOUT = '/opt/ibm/powerkvm/ui/backend/config/partition.layout'

# PowerKVM functions
POWERKVM_FUNCTIONS = '/opt/ibm/powerkvm/powerkvm-functions'

# Path to default yaboot.conf file
DEFAULT_YABOOT_CONF = '/opt/ibm/powerkvm/config/yaboot.conf'

# path to tarball file that contains the PowerKVM repo
TARBALL_REPO = '/mnt/powerkvm-repo.tar.gz'

# path to the PowerKVM repo
POWERKVM_REPO = '/tmp/repo'

# Repo config
REPO_CONFIG = '/opt/ibm/powerkvm/config/powerkvm.repo'

# Path to the script that update system packages
UPDATE_PKGS_SCRIPT = '/opt/ibm/powerkvm/ui/backend/updatepackages.py'

# Cow backup
COW_BACKUP = 'cow-bkp.tar.gz'

# PowerKVM logs directory
POWERKVM_LOG_DIR = "/var/log/powerkvm"

# Log format
LOG_FORMATTER = '%(asctime)s %(levelname)s %(pathname)s:%(funcName)s:%(lineno)d: %(message)s'

# LiveCD installer log
LIVECD_INSTALLER_LOG = "/tmp/installer.log"

# System image installer log
SYSIMG_INSTALLER_LOG = "%s/installer.log" % POWERKVM_LOG_DIR

# LiveCD partitioner log
LIVECD_PARTITIONER_LOG = "/tmp/partitioner.log"

# System image partitioner log
SYSIMG_PARTITIONER_LOG = "%s/partitioner.log" % POWERKVM_LOG_DIR

# Error archive
ERROR_ARCHIVE_PATH = "/tmp/ibmpkvm_log.tar.gz"

# Error log path
ERROR_LOG_PATH = "/tmp/ibmpkvm_error.log"

INSTALL_ROOT_DIR="/tmp/pkvm_install"

MIN_DISK_SIZE = '40GiB'

# Kickstart file
KICKSTART_FILE = "%s/kop.ks" % (POWERKVM_LOG_DIR)

# Kernel command line
KERNEL_CMDLINE_PATH = "/proc/cmdline"

# prep/boot partitions sizes
PREP_SIZE = '8MiB'
BOOT_SIZE = '512MiB'

# logical volumes
LV_PREFIX = 'ibmpkvm_'
LV_DATA = 'lv_data'
LV_DATA_MIN_SIZE = '1'
LV_SWAP = 'lv_swap'
LV_SWAP_MIN_SIZE = '8'
LV_SYSTEM = 'lv_system'
LV_SYSTEM_MIN_SIZE = '20'
LV_VAR = 'lv_var'
LV_VAR_MIN_SIZE = '10'
LVS_METADATA = {
    LV_SYSTEM: {"minSize": LV_SYSTEM_MIN_SIZE,
                "mountPoint": "/",
                "fsCheckPriority": 1,
                "fsType": "ext4",
                "devBackup": 1},
    LV_DATA:   {"minSize": LV_DATA_MIN_SIZE,
                "mountPoint": "/var/lib/libvirt/images",
                "fsCheckPriority": 2,
                "fsType": "ext4",
                "devBackup": 1},
    LV_SWAP:   {"minSize": LV_SWAP_MIN_SIZE,
                "mountPoint": "swap",
                "fsCheckPriority": 0,
                "fsType": "swap",
                "devBackup": 0},
    LV_VAR:    {"minSize": LV_VAR_MIN_SIZE,
                "mountPoint": "/var",
                "fsCheckPriority": 2,
                "fsType": "ext4",
                "devBackup": 1},
}
REMAINING_SIZE_LV = LV_DATA
LVS_UI_ORDER = [LV_SYSTEM, LV_SWAP, LV_VAR, LV_DATA]

PKVM2_VG_ROOT = 'vg_root'
PKVM2_VG_LOG  = 'vg_log'
PKVM2_VG_SWAP = 'vg_swap'
PKVM2_VG_DATA = 'vg_data'

PKVM2_LV_ROOT = 'lv_root'
PKVM2_LV_LOG  = 'lv_log'
PKVM2_LV_SWAP = 'lv_swap'
PKVM2_LV_DATA = 'lv_data'

PKVM2_LVM_STRUCTURE = {LV_PREFIX + PKVM2_VG_ROOT : LV_PREFIX + PKVM2_LV_ROOT,
                       LV_PREFIX + PKVM2_VG_LOG : LV_PREFIX + PKVM2_LV_LOG,
                       LV_PREFIX + PKVM2_VG_SWAP : LV_PREFIX + PKVM2_LV_SWAP,
                       LV_PREFIX + PKVM2_VG_DATA : LV_PREFIX + PKVM2_LV_DATA}

# Used when preserving logical volumes in installation
PRESERVABLE_MOUNT_POINTS = {"pkvm2": ['/var/log',
                                      '/var/lib/libvirt/images'],
                            "pkvm3": [LVS_METADATA[LV_VAR]['mountPoint'],
                                      LVS_METADATA[LV_DATA]['mountPoint']]}

CLEANUP_DIRS = ['/var/cache/yum', '/var/lib/rpm', '/var/lib/yum', '/var/tmp']
