#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import os
import re
import shutil
import stat
import subprocess
from tempfile import mkdtemp


#
# CONSTANTS
#


#
# CODE
#
def getAllHardDisks():
    """
    Get all hard disks found in the system

    Returns:
        [str]: list of all the hard disks found in the system
    """
    result = []

    fd = open("/proc/partitions")
    content = fd.readlines()
    fd.close()

    for line in content:
        line = line.strip().split()

        if len(line) >= 4:
            dev = line[3]
            if ('hd' in dev or 'sd' in dev) and len(dev) == 3:
                result.append(dev)

    return result

def getUUID(partition):
    """
    Returns the partition UUID

    Args:
        partition (basestring): partition

    Returns:
        basestring: uuid
    """
    blkcmd = 'blkid -o value -s UUID %s' % partition
    proc = subprocess.Popen(blkcmd, shell=True, stdout = subprocess.PIPE)
    uuid = proc.communicate()[0]

    return uuid[:-1]

def getPartitions(disk):
    """
    Returns all partitions in a disk

    Args:
        disk (str): path to the disk [/dev/<name>]

    Returns:
        [str]: list of all partitions in a disk
    """
    fd = open("/proc/partitions")
    content = fd.read()
    fd.close()

    return re.findall("%s.+" % disk, content)

def isBlockDevice(disk):
    """
    Check if the disk is a block device

    Args:
        disk (str): path to the disk [/dev/<name>]

    Returns:
        bool: True if the disk is a block device; False otherwise
    """
    if not os.path.exists(disk):
        return False

    mode = os.stat(disk).st_mode
    return stat.S_ISBLK(mode)

def mount(partition, mpoint=None):
    """Mount a partition in a temporaty mount point if mpoint is None.
    If mpoint is specified, partition is mounted on it.

    Args:
        partition (str): partition name to be mounted
        mpoint (str): destination mount point (optional)

    Returns:
        string: None if the parition could not be mounted; or the mount
            point directory where partition was successfully mounted
    """

    if mpoint == None:
        mountDir = mkdtemp()
    else:
        mountDir = mpoint

    cmd = "mount %s %s >/dev/null 2>&1" % (partition, mountDir)

    rc = subprocess.call(cmd, shell=True)

    if rc == 1:
        return None

    return mountDir

def umount(directory, removeMountPoint=True):
    """
    Umount some mount point directory

    Args:
        directory (str): mount point to be umounted
        removeMountPoint (bool): True to remove the directory used to mount
            the device. False otherwise.

    Returns:
        bool: True if the directory was successfully umounted; False otherwise
    """
    cmd = "umount %s &>/dev/null" % directory
    rc = subprocess.call(cmd, shell=True)

    if rc == 1:
        # kill all processes that are using the directory and try to umount it again
        subprocess.call("fuser -km %s > /dev/null 2>&1" % directory, shell=True)

        rc = subprocess.call(cmd, shell=True)
        if rc == 1:
            return False

    if removeMountPoint:
        shutil.rmtree(directory, ignore_errors=True)

    return True

def mountEnvironment(basedir):
    """
    Mount system directories for installing

    Args:
        basedir (str): chroot dir
    """
    # make sure all necessary directories exists
    if not os.path.isdir('%s/proc' % basedir):
        os.makedirs('%s/proc' % basedir)

    if not os.path.isdir('%s/dev' % basedir):
        os.makedirs('%s/dev' % basedir)

    if not os.path.isdir('%s/sys' % basedir):
        os.makedirs('%s/sys' % basedir)

    if not os.path.isdir('%s/run' % basedir):
        os.makedirs('%s/run' % basedir)

    if not os.path.isdir('%s/dev/pts' % basedir):
        os.makedirs('%s/dev/pts' % basedir)

    if not os.path.isdir('%s/dev/shm' % basedir):
        os.makedirs('%s/dev/shm' % basedir)

    if not os.path.isdir('%s/sys/fs/selinux' % basedir):
        os.makedirs('%s/sys/fs/selinux' % basedir)

    subprocess.call('mount -o bind /proc %s/proc' % basedir , shell=True)
    subprocess.call('mount -o bind /dev %s/dev' % basedir , shell=True)
    subprocess.call('mount -o bind /sys %s/sys' % basedir , shell=True)
    subprocess.call('mount -o bind /run %s/run' % basedir , shell=True)
    subprocess.call('mount -t devpts -o gid=5,mode=620 devpts %s/dev/pts' % basedir , shell=True)
    subprocess.call('mount -t tmpfs -o defaults tmpfs %s/dev/shm' % basedir , shell=True)
    subprocess.call('mount -t selinuxfs -o defaults selinuxfs %s/sys/fs/selinux > /dev/null 2>&1' % basedir , shell=True)

def umountEnvironment(basedir):
    """
    Umount system directories

    Args:
        basedir (str): chroot dir
    """
    subprocess.call('umount %s/sys/fs/selinux > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/dev/shm > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/dev/pts > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/run > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/sys > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/dev > /dev/null 2>&1' % basedir , shell=True)
    subprocess.call('umount %s/proc > /dev/null 2>&1' % basedir , shell=True)
