#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import os
import re
import subprocess
import yum
import sys
import traceback
import urllib2

from yum import logging
from yum import logginglevels

from disk import umount
from config import *

from callbacks.downloadcallback import DownloadCallback
from callbacks.installationcallback import InstallationCallback
from callbacks.transactioncallback import TransactionCallback

from modules.identifysystem.identifysystem import IdentifySystem
from controller.pkvmerror import PKVMError


#
# CONSTANTS
#
DEFAULT_REPO_PATH = 'file:///media/packages/'
INSTALLER_PATH = 'opt/ibm/powerkvm/'
DVD_LABEL = '/dev/disk/by-label/POWERKVM_LIVECD'
NFS_LOCAL_REPO = '/var/kop'
PKVM_GPGKEY = '/etc/pki/rpm-gpg/RPM-GPG-KEY-ibm_powerkvm'
GROUPS_TO_INSTALL = ["Base", "Virtualization"]

#
# CODE
#
class Installer():

    def __init__(self):
        """
        Constructor
        """
        pass

    def installPackages(self, rootDir, callback):
        """
        Installs all packages of given repository on a system root directory.

        Args:
            rootDir (str): path to temporary folder mounting root device
            callback (method): method to update installation progress
        """
        logger = logging.getLogger(__name__)

        # create temporary folder inside root directory
        basecache = os.path.join(rootDir, 'tmp')
        os.makedirs(basecache)

        # get repository according user preference
        repo = self.__getRepo()
        netRepo = False

        # no repo found: panic

        if repo is None:
            logger.critical('No repository found in the system')
            raise PKVMError("031")

        # repo is different from default: set a flag indicating
        if repo != DEFAULT_REPO_PATH and repo != 'file://' + NFS_LOCAL_REPO:
            netRepo = True
            repodata_file = self.__getRepodataFile(repo, logger)
            if not self.__checkSHA1(repodata_file, logger):
                raise PKVMError("033")

        # fixme: rpm lib writes directly in stdout and breaks the newt screen, we need to
        # import the key before yum tries to use it.
        subprocess.call('rpm --root=%s --initdb' % rootDir, shell=True)
        subprocess.call('rpm --root=%s --import %s' % (rootDir, PKVM_GPGKEY), shell=True)

        # configure yum logger
        yum.logginglevels._added_handlers = True

        yumLogger = yum.logging.getLogger('yum')
        yumLogger.setLevel(logging.DEBUG)
        yumLoggerHandler = yum.logging.FileHandler(filename='/tmp/yum.log', mode='w')
        yumLoggerHandler.setLevel(yum.logging.DEBUG)
        yumLogger.addHandler(yumLoggerHandler)

        # instantiate yum object
        yb = yum.YumBase()

        # disable all repos
        yb.repos.disableRepo('*')

        try:
            yb.pkgSack
        except:
            raise PKVMError("034")

        # configure yum parameters
        yb.conf.installroot = rootDir
        yb.conf.assumeyes = True
        yb.conf.cache = 0
        yb.conf.basecachedir = basecache
        # where to write yum metadata
        yb.conf.persistdir = rootDir + yb.conf.persistdir

        # dont check gpg locally
        yb.conf.gpgcheck = True
        if self.__isInternalBuild():
            yb.conf.gpgcheck = False

        # configure callback to watch installation progress
        downloadCallback = DownloadCallback(callback)
        yb.repos.setProgressBar(downloadCallback)

        # local repository to install packages: mount it
        if not netRepo:
            cmd = "mount -o loop %s /media 2>/dev/null" % DVD_LABEL

            # can't mount disk: error
            if subprocess.call(cmd, shell=True) == 1:
                raise PKVMError("048")

        # create a temporary rw folder to yum cache
        cmd = "mount -t ramfs -o size=20m ramfs /var/lib/yum 2>/dev/null"

        # can't create yum cache: error
        if subprocess.call(cmd, shell=True) == 1:
            raise PKVMError("035")

        # create a temporary repo to point to frobisher packages
        newrepo = yum.yumRepo.YumRepository('frobisher')

        # dont check gpg locally
        if not self.__isInternalBuild():
            newrepo.gpgcheck = True
            newrepo.gpgkey = 'file://%s' % PKVM_GPGKEY

        newrepo.name = 'frobisher'
        newrepo.baseurl = [repo]
        newrepo.basecachedir = basecache

        # IMPORTANT: This attribute MUST be set to True so that we are able
        # to see the groups present in this repository.
        newrepo.enablegroups = True

        yb.repos.add(newrepo)
        yb.repos.enableRepo(newrepo.id)
        yb.doRepoSetup(thisrepo=newrepo.id)

        # Mark groups to install
        yb.repos.populateSack()
        yb.pkgSack.buildIndexes()

        for group_name in GROUPS_TO_INSTALL:
            for grp in yb.comps.return_groups(group_name):
                yb.selectGroup(grp.groupid)

        # execute installation
        try:
            yb.resolveDeps()
            yb.buildTransaction()
            # Get number of transaction members after we have resolved dependencies.
            totalPkgs = len(yb.tsInfo.getMembers())
            yb.processTransaction(
                TransactionCallback(downloadCallback),
                None,
                InstallationCallback(callback, totalPkgs))

        except Exception, e:
            logger.critical('Error installing packages: %s' % str(e))
            raise PKVMError("037")

        yb.igroups.save()

        # close yum instance
        yb.closeRpmDB()
        yb.close()

        # can't umount yum cache: error
        if not umount('/var/lib/yum', False):
            raise PKVMError("038")
        # local repo: umount media
        if not netRepo:

            # can't umount media: error
            if not umount('/media', False):
                raise PKVMError("039")

    def __getRepoUrl(self):
        """
        Get the repo URL from the boot parameter

        Returns:
            str: repo from cmdline or None if not found
        """
        # open /proc/cmdline and get its content
        fd = open('/proc/cmdline')
        content = fd.read()
        fd.close()

        # split options in a array
        options = content.split()

        # look for repo= option
        for option in options:

            # not the option: skip
            if not 'repo=' in option:
                continue
            # remove repo= from the param
            return option[5:]
        return None

    def __getRepoLocal(self):
        """
        Get the repo URL from NFS boot

        Returns:
            str: repo from cmdline or None if repo not found
        """
        # exclusive directory is found: return it as our repo
        if os.path.isdir(NFS_LOCAL_REPO):
            return "file://%s" % NFS_LOCAL_REPO

        return None


    def __getRepoDVD(self):
        """
        Get the repo from the DVD

        Returns:
            str: repo from cmdline or None if repo not found
        """
        # DVD is inserted in drive: return the repo name
        if os.path.islink(DVD_LABEL):
            return DEFAULT_REPO_PATH

        return None


    def __getRepo(self):
        """
        Get the repository to install the system

        Returns:
            str: repo from cmdline or None if repo not found
        """
        # repo found in boot param: return it
        repo = self.__getRepoUrl()
        if repo:
            return repo

        # repo found in nfs mount location: return it
        repo = self.__getRepoLocal()
        if repo:
            return repo

        # kvm on power DVD in the drive: return it
        repo = self.__getRepoDVD()
        if repo:
            return repo

        return None


    def __checkSHA1(self, repofile, logger):

        logger.info('CheckSHA1')
        SHA1_LIVECD = '/primary-livecd.sha1'
        SHA1_REPO = '/tmp/primary.sha1'

        f = open(SHA1_LIVECD, 'rb')
        str1 = f.read()
        f.close()

        os.popen('sha1sum  /tmp/%s > %s 2>/dev/null' % (repofile, SHA1_REPO))

        with open(SHA1_REPO, 'rb') as f:
            return f.read().split()[0] == str1.split()[0]


    def __getRepodataFile(self, url, logger):
        """
        Downloads a repodata file that will be used to check
        the repo signature.

        Args:
            url (str): repo url
            logger (logging): logger object

        Returns:
            str: filename of the downloaded file
        """
        logger.info('Get repodata_file ')

        # try to open the url
        try:
            file = urllib2.urlopen(url + '/repodata/')

        # raise 'invalid repo' if it's not possible to open the repo
        except Exception, e:
            raise PKVMError("032")

        splitLines = file.read().splitlines()

        # check if the necessary file is present
        found = None
        for line in splitLines:
            if re.search('-primary.sqlite.bz2', line):
                found = line

        # raise an exception if the requested file is not found
        if not found:
            logger.critical('-primary.sqlite.bz2 not found in repo %s' % url)
            raise PKVMError("094")

        # get only the filename
        filename = re.split('([\d\w]+-primary.sqlite.bz2)', found)

        repodata_file = url + '/repodata/' + filename[1]

        logger.info('Download repodata_file')

        # Use subprocess module for calling wget.
        # 'shell = False' parameter enforces that no shell expansion wil be done to
        # the command.
        proc = subprocess.Popen(['wget', '-P', '/tmp/', repodata_file],
                                stderr = subprocess.PIPE, shell = False)
        _, err = proc.communicate()

        # Check if we failed to download file.
        if not os.path.isfile(os.path.join('/tmp', filename[1])):
            logger.critical('Failed to download repodata file from \
                            {}'.format(repodata_file))
            logger.critical('{}'.format(err))
            raise PKVMError("052")

        return filename[1]


    def __isInternalBuild(self):
        """
        Checks whether this running system is an internal build or not

        Returns:
            bool: True for internal build, False otherwise
        """
        return os.path.isfile('/etc/.dont_verify_gpg')


    def __getSelinux(self):
        # open /proc/cmdline and get its content
        fd = open('/proc/cmdline')
        content = fd.read()
        fd.close()

        # split options in a array
        options = content.split()

        # if nothing selinux=1
        ret = True

        # look for selinux=0 or selinux=1 option
        for option in options:
            if option.lower() == 'selinux=1':
                ret = True

            if option.lower() == 'selinux=0':
                ret = False

        return ret


    def doPostInstall(self, rootDir, partitioner):
        """
        Executes some routines to configure the fresh installed system properly.

        Args:
            rootDir (str): path to temporary folder mounting root device
            partitioner (storage.partitioner.Partitioner): path to temporary folder mounting root device
        """
        logger = logging.getLogger(__name__)

        # finishes possible old transactions in the installed system
        subprocess.call('rm -f %s/var/lib/yum/transaction* > /dev/null 2>&1' % rootDir, shell=True)
        subprocess.call('chroot %s rpm --rebuilddb > /dev/null 2>&1' % rootDir, shell=True)

        # command to update root password on installed system
        cmd = "chroot %s passwd -d root > /dev/null" % rootDir

        # update password failed: error
        if subprocess.call(cmd, shell=True) != 0:
            raise PKVMError("040")

        # selinux=0 autorelabel on reboot
        if not self.__getSelinux():

            # create a file to relabel after reboot
            cmd = "chroot %s touch /.autorelabel" % rootDir

            # SELinux restoration failed: error
            if subprocess.call(cmd, shell=True) != 0:
                logger.error('Error touching autorelabel: %s' % sys.stderr)
                raise PKVMError("041")

            # command to restore SELinux configurations
            cmd = "chroot %s setfiles /etc/selinux/targeted/contexts/files/file_contexts /" % rootDir

            if subprocess.call(cmd, shell=True) != 0:
                logger.error('Error running setfiles: %s' % sys.stderr)
                raise PKVMError("042")

        # install Grub
        partitioner.installPRePBootloader()

        # configure fstab
        partitioner.updateFstab()

        # setup multipath
        partitioner.setupMultipath()


    def isEntitled(self, data):
        """
        Verify entitlement

        Returns:
            bool: True if is entitled
        """
        # FIXME: when entitlement method has been defined by the brand
        # implement it here
        # raise PKVMError("049")
        # raise PKVMError("050")
        # raise PKVMError("051")

        return True

    def validateOpal(self, data):
        """
        Check if OPAL firmware is running in target system

        Args:
            data (model.model): target system data
        """

        idsys = IdentifySystem()
        opal = idsys.isRunningOpal(data)
        if opal is False:
            raise PKVMError("051")

    def validateEntitlement(self, logger, model):

        logger.info("Checking entitlement...")
        if not self.isEntitled(model):
            logger.critical("Hardware is not entitled, aborting installation!")
            # @IMPROVEENT review PKVMError attributes
            raise PKVMError("003")
