#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import logging
import os
import shutil
import time
import traceback
import urllib

from controller.pkvmerror import PKVMError
from model.config import *
from modules.scriptbase import ScriptBase


#
# CONSTANTS
#
MARK_FILE = ".ibm_kop"
DONT_VERIFY_OPAL = "/etc/.dont_verify_opal"


#
# CODE
#
class IdentifySystem(ScriptBase):
    """
    Identify if system has the OPAL firmware installed
    """

    def __init__(self):
        """
        Contructor
        """
        self.__logger = logging.getLogger(__name__)
        self.__logger.setLevel(logging.DEBUG)

    def onPreInstall(self, model):
        """
        Handles the pre install event. Identify if system has the OPAL
            firmware installed.

        Args:
            model (Model): installation data
        """
        try:
            self.__logger.debug("onPreInstall() called")

            self.detectOpal(model)
        except Exception as e:
            raise PKVMError("085")

    def detectOpal(self, model):
        """
        Search for string OPAL in /proc/cpuinfo and if string exists,
            fill model object accordingly.  If the control file
            exists, do not search for the string.
        """
        # assume we are not running on OPAL
        self.__setOpal(model, False)

        # verify control file
        if os.path.exists(DONT_VERIFY_OPAL):
            self.__logger.debug("detectOpal(): control file exists")
            self.__setOpal(model, True)
            return

        # search OPAL in /proc/cpuinfo
        with open("/proc/cpuinfo") as f:
            for line in iter(f.readline, ''):
                if "OPAL" in line:
                    self.__logger.debug("detectOpal(): OPAL found in /proc/cpuinfo")
                    self.__setOpal(model, True)
                    return
                if "qemu" in line:
                    if "pSeries" in line:
                        self.__logger.debug("detectOpal(): pSeries qemu found in /proc/cpuinfo")
                        self.__setOpal(model, True)
                        return

        self.__logger.debug("detectOpal(): OPAL = False")

    def isRunningOpal(self, model):
        """
        Return True if running on OPAL.  Return False otherwise.

        Returns:
            bool: True if on OPAL;  otherwise, False.
        """
        retval = False
        opalval = model.get('opal')

        if opalval == True:
            retval = True

        self.__logger.debug("isRunningOpal(): %s" % opalval)

        return retval

    def __setOpal(self, model, value=False):
        """
        Set OPAL boolean value.
        """
        self.__logger.debug("__setOpal(): %s" % value)

        model.insert('opal', value)
