#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import logging
import re
import traceback

from controller.pkvmerror import PKVMError
from modules.scriptbase import ScriptBase


#
# CONSTANTS
#
CONFIG_FILE="/etc/resolv.conf"


#
# CODE
#
class DNS(ScriptBase):
    """
    Edit the DNS configuration on the system
    """

    def __init__(self):
        """
        Constructor
        """
        self.__configFile = self.__parseConfigFile()
        self.__logger = logging.getLogger(__name__)
        self.__logger.setLevel(logging.DEBUG)

    def onPostInstall(self, model):
        """
        Handles the post install event

        Args:
            model (dict): installation data
        """
        try:
            mountDir = model.get('mountDir')

            # no dns to be configured, return
            if not model.get('dns'):
                return

            dns = model.get('dns')

            # update resolv.conf
            resolv = mountDir + CONFIG_FILE
            with open(resolv, 'wt') as f:

                for ns in dns['dnsList']:
                    f.write('nameserver {}\n'.format(ns))

                if dns['search']:
                    f.write('search {}\n'.format(dns['search']))

        except Exception as e:
            raise PKVMError("090")


    def __parseConfigFile(self):
        """
        Parse CONFIG_FILE and set internal attribute that holds the
            configuration read.
        """
        conf = {'nameservers': []}

        try:
            with open(CONFIG_FILE, 'rt') as f:
                content = f.readlines()

            for line in content:
                line = line.strip()
                if line.startswith(";") or line.startswith("#") or not line:
                    continue

                key = line.split()[0].strip()
                value = line.split(key)[1].strip()

                if key == "nameserver":
                    conf['nameservers'].append(value)
                else:
                    conf[key] = value
        except IOError:
            # @TODO: use KernelCmdLine class
            # Failed to read resolv.conf - try to read nameservers from cmdline
            with open('/proc/cmdline', 'rt') as f:
                content = f.read()
            regex = re.compile(r'nameserver=([A-Za-z0-9\.\-]+)[ \n]?')
            conf['nameservers'] = regex.findall(content)
        return conf

    def writeConfigFile(self):
        """
        Write a new configuration file based on the config dictionary.
        """

        with open(CONFIG_FILE, 'wt') as f:
            for ns in self.__configFile['nameservers']:
                f.write('nameserver %s\n' % ns)

            if 'search' in self.__configFile:
                f.write('search %s\n' % self.__configFile['search'])

    def getNameservers(self):
        """
        Returns a list of nameservers (strings).

        Returns:
            [str]: list of nameservers
        """
        return self.__configFile['nameservers']

    def setNameservers(self, nameservers):
        """
        Sets the list of the nameservers to the current configuration
            dictionary.

        Args:
            nameservers ([str]): list of nameservers

        Returns:
            bool: False if the list of nameservers is not valid; True,
                otherwise.
        """
        if not nameservers:
            return False

        for ns in nameservers:
            if not isinstance(ns, basestring):
                return False

        self.__configFile['nameservers'] = nameservers
        return True

    def setSearchList(self, value):
        """
        Sets the search list

        Args:
            value (str): search list
        """
        self.__configFile["search"] = value

    def getSearchList(self, default = ""):
        """
        Returns the search list

        Returns:
            str: search list
        """
        searchList = self.__configFile.get("search")

        if searchList is not None:
            return searchList

        return default
