#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from subprocess import Popen, PIPE
from time import sleep
import fileinput
import os
import re
import shlex
import shutil

import ethtool

from model.config import KERNEL_CMDLINE_PATH
from modules.network import utils

#
# CONSTANTS
#
ETHERNET_TYPE = '1'


#
# CODE
#
class Network:
    """
    Utilities to get network information
    """

    def __new__(cls):
        raise NotImplementedError("This class cannot be instantiated.")

    def getVirtualNic():
        """
        Returns all virtual network interfaces available on the system

        Returns:
            [str]: list of Virtual NIC's
        """
        proc = Popen(['ls', '/sys/devices/virtual/net'], stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            raise Exception("NETWORK: Failed to list virtual net devices (exit code = %s):\n%s" % (proc.returncode, err))
        return out.rstrip().split("\n")
    # getVirtualNic
    getVirtualNic = staticmethod(getVirtualNic)

    def isEthernet(nic):
        """
        Checks whether a given network interface uses Ethernet
            protocol in the link layer.

        Args:
            nic (str): network device

        Returns:
            bool: True if it is Ethernet
        """
        file = None

        try:
            file = open('/sys/class/net/%s/type' % nic, 'rb')

        # unable to open the nic type for any reason, return False
        except:
            return False

        isEthernetType = False
        if file.read().rstrip() == ETHERNET_TYPE:
            isEthernetType = True

        file.close()

        return isEthernetType
    isEthernet = staticmethod(isEthernet)

    def getCmdLineNIC():
        """
        Return a list of network interfaces names specified in kernel
            command line.

        Returns:
            [str]: Network interfaces names

        """
        # The list of configured interface names, i.e. the ones
        # specified in kernel command line.
        configuredNics = []

        # Read content of kernel command line.
        kernelCmdline = ""
        with open(KERNEL_CMDLINE_PATH) as f:
            kernelCmdline = f.read().rstrip()

        # The kernel command line might have something like this:
        #
        #     ifname=net0:4C:45:42:45:14:01 ip=net0:dhcp ifname=net1:4C:45:42:45:14:02 ip=net1:dhcp
        #
        # Parse and obtain all interfaces names.
        ifName = None
        for i in kernelCmdline.split(" "):
            if i.startswith("ifname"):
                ifName = i.split("=")[1].split(":")[0]
                configuredNics.append(ifName)

        return configuredNics
    getCmdLineNIC = staticmethod(getCmdLineNIC)

    def getUdevPredictableName(dev):
        """
        Returns the mapped network interface name based on udev
            predictable rules.  Respected network interfaces predefined in
            boot command line.

        Returns:
            str: NIC name according UDEV predictable rules.
        """
        device = None
        nics = Network.getCmdLineNIC()
        if dev in nics:
            return dev

        strCmd1 = "udevadm info --query=property --path=/sys/class/net/%s"
        cmd2 = "grep ID_NET_NAME_PATH"
        cmd1 = strCmd1 % dev
        proc1 = Popen(cmd1.split(" "), stdout=PIPE)
        proc2 = Popen(cmd2.split(" "), stdin=proc1.stdout, stdout=PIPE)
        proc1.stdout.close()
        out, err = proc2.communicate()
        if proc2.returncode == 0:
            # Found udev predictable NIC name
            device = out.rstrip().split('=')[1]
        elif proc2.returncode == 1:
            # No udev predictable NIC name, use the same dev
            device = dev
        else:
            raise Exception("Failed to get predictable network name from udev "
                            "(exit code = %s):\n%s" % (proc2.returncode, err))

        return device
    getUdevPredictableName = staticmethod(getUdevPredictableName)

    def getLinkedInterfaces(liveDVD=True):
        """
        Return a list of all network interfaces on the system with
            active link (the same as ethtool.get_active_devices(),
            however, it also works when the system is booted via
            liveDVD and the network must be started manually -
            Network.startLiveDVDNetwork())

        Args:
            mountDir (bool): True is running in liveDVD

        Returns:
            [str]: list of NIC with active link
        """
        interfaces = []
        devices = ethtool.get_devices()
        for dev in devices:
            cmd = ['ethtool', dev]
            proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                # Here we verify if we really care about ethtool
                # error (ie is this really an ethernet NIC ?)
                if not Network.isEthernet(dev):
                    continue

                raise Exception("NETWORK: Failed to execute ethtool on %s (exit code = %s):\n%s" % (dev, proc.returncode, err))
            if out.rstrip().endswith("Link detected: yes"):
                if liveDVD:
                    device = Network.getUdevPredictableName(dev)
                else:
                    device = dev
                interfaces.append(device)
        return interfaces
    getLinkedInterfaces = staticmethod(getLinkedInterfaces)

    def getAvailableInterfaces(liveDVD=True):
        """
        Returns all the network interfaces (eth/en[pP]) available on
            the system

        Args:
            liveDVD (bool): True is running in liveDVD

        Returns:
            dict: (key = ethernet interface / value = mac addresss)
                available
        """
        interfaces = {}
        devices = ethtool.get_devices()
        virtualIfaces = Network.getVirtualNic()
        # TODO: Verify if we really shoud include virtual NICs in this list
        for dev in devices:

            # consider only NIC interfaces
            if not Network.isEthernet(dev):
                continue

            if dev not in virtualIfaces:
                if liveDVD:
                    device = Network.getUdevPredictableName(dev)
                else:
                    device = dev
                interfaces[device] = ethtool.get_hwaddr(dev)
        return interfaces
    getAvailableInterfaces = staticmethod(getAvailableInterfaces)

    def getActiveInterfaces(liveDVD=True):
        """
        Returns all the network interfaces (eth/en[pP] available on
            the system with network configured

        Args:
            mountDir (bool): True is running in liveDVD

        Returns:
            [str]: list of NIC with active link
        """
        interfaces = []
        devices = list(Network.getAvailableInterfaces(liveDVD).keys())
        for dev in devices:
            cmd1 = "ip address show %s" % dev
            proc1 = Popen(shlex.split(cmd1), stdout=PIPE, stderr=PIPE)
            cmd2 = "grep inet"
            proc2 = Popen(shlex.split(cmd2), stdin=proc1.stdout, stdout=PIPE, stderr=PIPE)
            proc1.stdout.close()
            output, err = proc2.communicate()
            if output.lstrip().startswith("inet "):
                if liveDVD:
                    device = Network.getUdevPredictableName(dev)
                else:
                    device = dev
                interfaces.append(device)
        return interfaces
    getActiveInterfaces = staticmethod(getActiveInterfaces)

    def deleteBrConfigFile(dev, mountDir=None):
        """
        Bring down, delete network bridge and delete its configuration
            file

        Args:
            configFile (dict): network device which bridge will be
                deleted
            mountDir (str): mounted directory if is in auto mode
        """
        bridge = "br%s" % dev
        ifcfgFile = "/etc/sysconfig/network-scripts/ifcfg-%s" % bridge
        if mountDir:
            ifcfgFile = mountDir + ifcfgFile
        if os.path.exists(ifcfgFile):
            os.remove(ifcfgFile)

        # Bring down and delete bridge interface
        cmdDown = ['ip', 'link', 'set', bridge, 'down']
        cmdDelete = ['brctl', 'delbr', bridge]
        if mountDir:
            cmdDown.insert(0, mountDir)
            cmdDown.insert(0, 'chroot')
            cmdDelete.insert(0, mountDir)
            cmdDelete.insert(0, 'chroot')

        proc = Popen(cmdDown, stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        proc = Popen(cmdDelete, stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
    deleteBrConfigFile = staticmethod(deleteBrConfigFile)

    def writeConfigFile(configFile, mountDir=None):
        """
        Write the new configuration into the file

        Args:
            configFile (dict): content of network config file
            mountDir (str): mounted directory if is in auto mode
        """
        # remove double quotes from NIC name
        ifname = utils.removeDoubleQuotes(configFile['DEVICE'])
        ifcfgFile = "/etc/sysconfig/network-scripts/ifcfg-%s" % ifname
        if mountDir:
            ifcfgFile = mountDir + ifcfgFile

        line = "# Generated by IBM powerkvm installer\n"
        with open(ifcfgFile, "w") as fd:
            fd.write(line)
            for key, value in configFile.iteritems():
                line = "%s=%s\n" % (key, value)
                fd.write(line)

        # if configFile['DEVICE'].startswith("br") and configFile['ONBOOT'] == 'yes':
        #     ifcfgFile = "/etc/sysconfig/network-scripts/ifcfg-eth%s" % configFile['DEVICE'][2:]
        #     if not os.path.isfile(ifcfgFile):
        #         ifcfgFile = "/etc/sysconfig/network-scripts/ifcfg-enp%s" % configFile['DEVICE'][2:]
        #         if not os.path.isfile(ifcfgFile):
        #             ifcfgFile = "/etc/sysconfig/network-scripts/ifcfg-enP%s" % configFile['DEVICE'][2:]
        #     if mountDir:
        #         ifcfgFile = mountDir + ifcfgFile
        #     searchExp = "ONBOOT=no"
        #     replaceExp = "ONBOOT=yes"
        #     for line in fileinput.input(ifcfgFile, inplace=1):
        #         print line.replace(searchExp, replaceExp),
    writeConfigFile = staticmethod(writeConfigFile)

    def copyConfigFile(mountDir):
        """
        Copy configuration from ISO to installed system

        Args:
            configFile (str): mount path dir of installed system
        """
        if not mountDir:
            raise Exception("NETWORK: invalid system mount point")

        ifcfgDir = '/etc/sysconfig/network-scripts/'
        ifcfgDirInstalled = mountDir + ifcfgDir
        ifcfgFiles = [f for f in os.listdir(ifcfgDir) if re.match(r'ifcfg-.*', f)]
        # ifcfgFiles = filter(lambda f: re.match(r'ifcfg-.*', f), os.listdir(ifcfgDir))
        ifcfgFiles.remove('ifcfg-lo')

        for srcFile in ifcfgFiles:
            destFile = ifcfgDirInstalled + srcFile
            srcFile = ifcfgDir + srcFile
            shutil.copy(srcFile, destFile)

    copyConfigFile = staticmethod(copyConfigFile)

    def restartNetworkService(mountDir=None, nic=None):
        """
        Restart network service using sysmtectl (systemd)

        Args:
            mountDir (str): mounted directory if is in auto mode
            nic (str): network interface
        """
        # /etc/sysconfig/network is required for network service script
        networkFile = '/etc/sysconfig/network'
        if mountDir:
            networkFile = mountDir + networkFile
        if not os.path.isfile(networkFile):
            with open(networkFile, 'a'):
                os.utime(networkFile, None)

        # Restart network service
        if mountDir:
            # auto mode
            proc = Popen(['chroot', mountDir, '/sbin/chkconfig', 'network', 'on'], stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise Exception("NETWORK: Failed to enable network service (exit code = %s):\n%s" % (proc.returncode, err))
        elif nic:
            nics = [nic]
            bridge = "br%s" % nic
            ifcfgFilePattern = "/etc/sysconfig/network-scripts/ifcfg-%s"
            activeNics = Network.getLinkedInterfaces(False)
            # only restart nic if it has an active link
            if nic in activeNics or bridge in activeNics:
                ifcfgFile = ifcfgFilePattern % bridge
                if os.path.exists(ifcfgFile):
                    nics.append(bridge)

                for iface in nics:
                    proc = Popen(['ifdown', iface], stdout=PIPE, stderr=PIPE)
                    out, err = proc.communicate()
                    if proc.returncode != 0:
                        raise Exception("NETWORK: Failed to deactivate %s (exit code = %s):\n%s" % (iface, proc.returncode, err))

                    # Only start network config if ONBOOT=yes
                    ifcfgFile = ifcfgFilePattern % iface
                    onboot = "ONBOOT=yes"
                    if onboot in open(ifcfgFile).read():
                        proc = Popen(['ifup', iface], stdout=PIPE, stderr=PIPE)
                        out, err = proc.communicate()
                        if proc.returncode != 0:
                            raise Exception("NETWORK: Failed to activate %s (exit code = %s):\n%s" % (iface, proc.returncode, err))
        else:
            # manual mode
            # restart network service
            proc = Popen(['systemctl', 'restart', 'network.service'], stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise Exception("NETWORK: Failed to restart network service (exit code = %s):\n%s" % (proc.returncode, err))

            # enable network service
            proc = Popen(['systemctl', 'enable', 'network.service'], stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise Exception("NETWORK: Failed to enable network service (exit code = %s):\n%s" % (proc.returncode, err))
    restartNetworkService = staticmethod(restartNetworkService)

    def disableNIC(mountDir, nic):
        """
        Disable network interface (ONBOOT=no).

        Args:
            mountDir (str): mounted directory if is in auto mode
            nic (str): network interface
        """
        bridge = "br%s" % nic
        nics = [nic, bridge]
        ifcfgFilePattern = "/etc/sysconfig/network-scripts/ifcfg-%s"
        for iface in nics:
            ifcfgFile = ifcfgFilePattern % iface
            if mountDir:
                ifcfgFile = mountDir + ifcfgFile
            if os.path.isfile(ifcfgFile):
                searchExp = "ONBOOT=yes"
                replaceExp = "ONBOOT=no"
                for line in fileinput.input(ifcfgFile, inplace=1):
                    print line.replace(searchExp, replaceExp),
    disableNIC = staticmethod(disableNIC)

    def setNTP(mountDir):
        proc = Popen(['/bin/ln', '-s', '/usr/lib/systemd/system/ntpd.service', mountDir+'/etc/systemd/system/multi-user.target.wants/ntpd.service'], stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            raise Exception("NETWORK: Failed to enable ntpdate service (exit code = %s):\n%s" % (proc.returncode, err))
    setNTP = staticmethod(setNTP)

    def activeLinkInterfaces(interfaces):
        """
        Active link (ifconfig nic up) for the given NICs

        Args:
            interfaces ([str]): list of the NICs to be activated (link)
        """
        # Active NIC's link
        for iface in interfaces:
            cmd = ['ifconfig', iface, 'up']
            proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise Exception("NETWORK: Failed to activate link on liveDVD %s (exit code = %s):\n%s" % (cmd, proc.returncode, err))
        # wait link up
        sleep(5)
    activeLinkInterfaces = staticmethod(activeLinkInterfaces)

    def deactiveLinkInterfaces(interfaces):
        """
        Deactive link (ifconfig nic down) for the given NICs

        Args:
            interfaces ([str]): list of the NICs to be deactivated
                (link)
        """
        # Active NIC's link
        for iface in interfaces:
            cmd = ['ifconfig', iface, 'down']
            proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise Exception("NETWORK: Failed to deactivate link on liveDVD %s (exit code = %s):\n%s" % (cmd, proc.returncode, err))
        # wait link up
        sleep(5)
    deactiveLinkInterfaces = staticmethod(deactiveLinkInterfaces)

    def startLiveDVDNetwork():
        """
        Try to start network (dhcp) when boot from liveDVD.
        """
        # Get all NICs
        interfaces = Network.getAvailableInterfaces(False)
        # Do not reconfigure NIC defined by user in boot line
        nics = Network.getCmdLineNIC()
        for nic in nics:
            if interfaces.get(nic):
                del interfaces[nic]

        # Active NIC's link
        Network.activeLinkInterfaces(interfaces.keys())

        # Release NIC's
        cmd = ['dhclient', '-r']
        proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            raise Exception("NETWORK: Failed to release NIC's on liveDVD %s (exit code = %s):\n%s" % (cmd, proc.returncode, err))

        # Remove virtual NIC's
        virtualInterfaces = Network.getVirtualNic()
        activeInterfaces = Network.getLinkedInterfaces()
        for iface in virtualInterfaces:
            # We might have already removed a virtual NIC from
            # the active lists, so double check before removing
            if iface in activeInterfaces:
                activeInterfaces.remove(iface)

        # Try to configure network for each NIC's
        cmd = ['dhclient']
        cmd = cmd + activeInterfaces
        proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            raise Exception("NETWORK: Failed to configure network on liveDVD %s (exit code = %s):\n%s" % (cmd, proc.returncode, err))
        # wait 3 seconds per NIC
        sleep(len(activeInterfaces)*3)
    startLiveDVDNetwork = staticmethod(startLiveDVDNetwork)
