#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
import logging
import os
import pprint
import re
import traceback

from controller.pkvmerror import PKVMError
from modules.scriptbase import ScriptBase
from modules.network import utils
from modules.network.network import Network
from modules.network.networktopology import NetworkTopology

#
# CONSTANTS
#
IFCFG_FILE = "/etc/sysconfig/network-scripts/ifcfg-%s"


#
# CODE
#
class NetworkInterface(ScriptBase):
    """
    Represents a network interface

    IMPORTANT:
      __loadConfigFile() - Must be the first method to be executed
      setBridge() - Must be the second method to be executed
    """

    def __init__(self, name=None):
        """
        Contructor

        Args:
            name (str): network interface name (default = None)
        """
        self.__logger = logging.getLogger(__name__)
        self.__ethConfigFile = {
            'DEVICE': None,
            'TYPE': 'Ethernet',
            'HWADDR': None,
            'BOOTPROTO': 'none',
            'ONBOOT': 'no',
        }
        self.__brConfigFile = {
            'DEVICE': None,
            'TYPE': 'Bridge',
            'ONBOOT': 'no',
            'BOOTPROTO': 'none',
            'NM_CONTROLLED': 'no',
            'DELAY': '0',
        }
        self.__mountDir = None
        self.__hostname = None
        self.__hasBridge = False
        if name:
            self.__loadConfigFile(name)

    def __loadConfigFile(self, name):
        """
        Load previous network cfg file
        Must be executed before setBridge()

        Args:
            name (str): network interface card name (NIC)
        """
        self.__ethConfigFile = self.__parseConfigFile(name)
        if self.__ethConfigFile.get('BRIDGE'):
            self.setBridge(True)
            bridge = "br%s" % name
            self.__brConfigFile = self.__parseConfigFile(bridge)
        else:
            self.setBridge(False)

    def __parseConfigFile(self, dev):
        """
        Parse IFCFG_FILE into a dictionary

        Returns:
            dict: dictionary with the content file parsed
        """
        try:
            result = {}
            ifcfgFile = IFCFG_FILE % dev
            if self.__mountDir:
                ifcfgFile = self.__mountDir + ifcfgFile

            # If config file doesn't exist, we should create a template.
            if not os.path.exists(ifcfgFile):
                self.__logger.info('Missing config file for %s device.' % dev)
                self.__logger.info('Calling NetworkTopology to create all missing network config files.')
                devices_dict = Network.getAvailableInterfaces(liveDVD=False)
                active_devices = Network.getActiveInterfaces(liveDVD=False)
                net_topology = NetworkTopology()
                net_topology.genMissingNetConfig(devices_dict,
                                                 active_devices)

            with open(ifcfgFile) as fd:
                content = fd.readlines()

            for line in content:
                if line.startswith("#"):
                    continue
                key, value = line.strip().split("=")
                result[key] = value
            return result
        except IOError as e:
            self.__logger.critical("Failed to read network configuration file for %s device.",
                                   dev)
            raise PKVMError("027")
        except Exception as e:
            raise PKVMError("084")

    def setBridge(self, value=True):
        """
        Set if network bridge will be used or not.
        Must be executed after __loadConfigFile()

        Args:
            value (bool): True/False
        """
        self.__hasBridge = value

    def hasBridge(self):
        """
        Return if network bridge is used or not.

        Returns:
            bool: True if there network bridge is used; False, otherwise.
        """
        return self.__hasBridge

    def setDNS(self):
        """
        If DNS set exist on ifcfg file move it accordingly.
        """
        entriesMoved = []
        if self.__hasBridge:
            for k, v in self.__ethConfigFile.iteritems():
                if re.match('DNS[0-9]', k):
                    self.__brConfigFile[k] = v
                    entriesMoved.append(k)
            if entriesMoved:
                self.__logger.debug("Moving DNS entries from NIC to bridge:")
                self.__logger.debug("%s" % entriesMoved)
                for i in entriesMoved:
                    del self.__ethConfigFile[i]
        else:
            for k, v in self.__brConfigFile.iteritems():
                if re.match('DNS[0-9]', k):
                    self.__ethConfigFile[k] = v
                    entriesMoved.append(k)
            if entriesMoved:
                self.__logger.debug("Moving DNS entries from bridge to NIC:")
                self.__logger.debug("%s" % entriesMoved)
                for i in entriesMoved:
                    del self.__brConfigFile[i]

    def setDev(self, dev, macaddr):
        """
        Sets device name and mac address to the network interface and
            brigde if the last one is applicable

        Args:
            dev (str): device name
            macaddr (str): mac address
        """
        self.__ethConfigFile['DEVICE'] = dev
        self.__ethConfigFile['TYPE'] = 'Ethernet'
        if macaddr:
            self.__ethConfigFile['HWADDR'] = macaddr
        bridge = "br%s" % dev
        if self.__hasBridge:
            self.__ethConfigFile['BRIDGE'] = bridge
            self.__ethConfigFile['NM_CONTROLLED'] = 'no'
            self.__brConfigFile['DEVICE'] = bridge
            self.__brConfigFile['TYPE'] = 'Bridge'
            self.__brConfigFile['NM_CONTROLLED'] = 'no'
            self.__brConfigFile['DELAY'] = '0'
        else:
            if self.__ethConfigFile.get('BRIDGE'):
                del self.__ethConfigFile['BRIDGE']
            if self.__ethConfigFile.get('NM_CONTROLLED'):
                del self.__ethConfigFile['NM_CONTROLLED']

    def setOnBoot(self, value):
        """
        Set ONBOOT parameter

        Args:
            value (bool): ONBOOT state
        """
        if value:
            self.__ethConfigFile["ONBOOT"] = 'yes'
        else:
            self.__ethConfigFile["ONBOOT"] = 'no'
        if self.__hasBridge:
            self.__brConfigFile["ONBOOT"] = self.__ethConfigFile.get('ONBOOT')

    def getOnBoot(self):
        """
        Return ONBOOT state

        Returns:
            bool: True if ONBOOT value is 'yes'; False if ONBOOT value
                is 'no'.
        """
        enable = False
        if self.__hasBridge:
            if self.__brConfigFile.get("ONBOOT") == 'yes':
                enable = True
        else:
            if self.__ethConfigFile.get("ONBOOT") == 'yes':
                enable = True
        return enable

    def enableDHCP(self, flag):
        """
        Enable/Disable DHCP configuration.

        Args:
            flag (bool): True to enable DHCP; False otherwise
        """
        if flag:
            if self.__hasBridge:
                self.__brConfigFile["BOOTPROTO"] = "dhcp"
                for key in ["IPADDR", "NETMASK", "GATEWAY"]:
                    if key in self.__brConfigFile.keys():
                        del self.__brConfigFile[key]
                self.__ethConfigFile["BOOTPROTO"] = "none"
            else:
                self.__ethConfigFile["BOOTPROTO"] = "dhcp"

            for key in ["IPADDR", "NETMASK", "GATEWAY"]:
                if key in self.__ethConfigFile.keys():
                    del self.__ethConfigFile[key]
        else:
            if self.__hasBridge:
                self.__brConfigFile["BOOTPROTO"] = "none"
            self.__ethConfigFile["BOOTPROTO"] = "none"

    def setIp(self, value):
        """
        Sets IP address to the network interface

        Args:
            value (str): IP address
        """
        if self.__hasBridge:
            self.__brConfigFile["IPADDR"] = value
            if self.__ethConfigFile.get('IPADDR'):
                del self.__ethConfigFile['IPADDR']
        else:
            self.__ethConfigFile["IPADDR"] = value

    def getIp(self, default=""):
        """
        Returns the IP address

        Args:
            default (str): default value

        Returns:
            str: IP address or default value if it is None
        """
        if self.__hasBridge:
            ip = self.__brConfigFile.get("IPADDR")
        else:
            ip = self.__ethConfigFile.get("IPADDR")

        if ip is None:
            ip = default
        return utils.removeDoubleQuotes(ip)

    def setNetmask(self, value):
        """
        Sets network mask address to the network interface

        Args:
            value (str): network mask address
        """
        if self.__hasBridge:
            self.__brConfigFile["NETMASK"] = value
            if self.__ethConfigFile.get('NETMASK'):
                del self.__ethConfigFile['NETMASK']
        else:
            self.__ethConfigFile["NETMASK"] = value

    def getNetmask(self, default=""):
        """
        Returns the network mask

        Args:
            default (str): default value

        Returns:
            str: network mask address or default value if it is None
        """
        if self.__hasBridge:
            netmask = self.__brConfigFile.get("NETMASK")
        else:
            netmask = self.__ethConfigFile.get("NETMASK")

        if netmask is None:
            netmask = default
        return utils.removeDoubleQuotes(netmask)

    def setGateway(self, value):
        """
        Sets gateway to the network interface

        Args:
            value (str): gateway
        """
        if self.__hasBridge:
            self.__brConfigFile["GATEWAY"] = value
            if self.__ethConfigFile.get('GATEWAY'):
                del self.__ethConfigFile['GATEWAY']
        else:
            self.__ethConfigFile["GATEWAY"] = value

    def getGateway(self, default=""):
        """
        Returns the gateway

        Args:
            default (str): default value

        Returns:
            str: gateway or default value if it is None
        """
        if self.__hasBridge:
            gateway = self.__brConfigFile.get("GATEWAY")
        else:
            gateway = self.__ethConfigFile.get("GATEWAY")

        if gateway is None:
            gateway = default
        return utils.removeDoubleQuotes(gateway)

    def setHostname(self, value):
        """
        Sets hostname

        Args:
            value (str): hostname
        """
        self.__hostname = value

    def getHostname(self, default=""):
        """
        Returns the hostname

        Args:
            default (str): default value

        Returns:
            str: hostname or default value if it is None
        """
        hostname = self.__hostname

        if hostname is not None:
            return hostname

        return default

    def getBootProto(self):
        """
        Retuns the boot protocol

        Returns:
            str: boot protocol associated to the network interface
        """
        if self.__hasBridge:
            bootproto = self.__brConfigFile.get("BOOTPROTO")
        else:
            bootproto = self.__ethConfigFile.get("BOOTPROTO")
        return bootproto

    def getEthConfigFile(self):
        """
        Returns the content of network intergace config file

        Returns:
            dict: dictionary with the content of network interface
                config file
        """
        return self.__ethConfigFile

    def getBrConfigFile(self):
        """
        Returns the content of network bridge config file

        Returns:
            dict: dictionary with the content of network bridge config file
        """
        if self.__hasBridge:
            return self.__brConfigFile
        else:
            return None

    def setNoDefRoute(self, noDefRoute):
        """
        Do not allow the interface being set as the default route.

        Args:
            noDefRoute (bool):
        """
        if noDefRoute:
            if self.__hasBridge:
                self.__brConfigFile["DEFROUTE"] = "no"
                self.__brConfigFile["PEERROUTES"] = "no"
                if self.__ethConfigFile.get('DEFROUTE'):
                    del self.__ethConfigFile['DEFROUTE']
                if self.__ethConfigFile.get('PEERROUTES'):
                    del self.__ethConfigFile['PEERROUTES']
            else:
                self.__ethConfigFile["DEFROUTE"] = "no"
                self.__ethConfigFile["PEERROUTES"] = "no"

    def __saveIfcfgFile(self):
        """
        Save ifcfg file for current NIC and for its bridge, if it is
            applicable.
        """
        # Saving network cfg file
        self.__logger.debug("Writing on ISO image ethConfigFile:")
        for line in pprint.pformat(self.__ethConfigFile).split('\n'):
            self.__logger.debug(line)
        Network.writeConfigFile(self.__ethConfigFile)
        if self.__hasBridge:
            self.__logger.debug("Writing on ISO image brConfigFile:")
            for line in pprint.pformat(self.__brConfigFile).split('\n'):
                self.__logger.debug(line)
            Network.writeConfigFile(self.__brConfigFile)
        else:
            Network.deleteBrConfigFile(self.__ethConfigFile['DEVICE'])

    def __setNIC(self, model):
        """
        Create ifcfg file based on kickstart.

        Args:
            model (Model): installation data
        """
        self.__logger.debug("Setting NIC's according kickstart.")
        # userdefine
        for network in model.get('network'):
            # validate device
            if network.get('device'):
                #dev = network['device']
                #device = Network.getUdevPredictableName(dev)
                device = network['device']
            else:
                self.__logger.critical("Missing interface name in kickstart file.")
                raise PKVMError("028")
            # load previous configuration.
            self.__loadConfigFile(device)
            # bridge
            if network.get('bridge'):
                self.setBridge(True)
            # device
            self.setDev(device, None)
            # onboot
            if not network.get('onboot') is None:
                self.setOnBoot(network['onboot'])
            # dns
            self.setDNS()
            # dhcp
            if network['bootProto'] == "dhcp" and not network['ip']:
                self.enableDHCP(True)
            else:
                self.enableDHCP(False)
                if network.get('ip'):
                    self.setIp(network['ip'])
                if network.get('gateway'):
                    self.setGateway(network['gateway'])
                if network.get('netmask'):
                    self.setNetmask(network['netmask'])
                if network.get('hostname'):
                    self.setHostname(network['hostname'])
            self.setNoDefRoute(network['nodefroute'])

            # Saving network cfg file
            self.__saveIfcfgFile()

    def __setAllDHCP(self):
        """
        Config all NIC's to DHCP.
        """
        self.__logger.debug("Setting all NIC's to DHCP.")

        # Verify if there is a NIC redefined by bootline
        ifname = Network.getCmdLineNIC()
        if ifname is not None:
            # load previous configuration.
            self.__loadConfigFile(ifname)
            # set redefined NIC in bootline to DHCP
            self.enableDHCP(True)

            # Saving network cfg file
            self.__saveIfcfgFile()

    def onPreInstall(self, model):
        """
        Handles the pre install event

        Args:
            model (Model): installation data
        """
        try:
            self.__logger.info("Executing NetworkInterface module (EVT_PRE_INSTALL).")
            networkData = model.get('network')
            if networkData:
                if (len(networkData) == 1 and
                        networkData[0].get('device') == '' and
                        networkData[0].get('bootProto') == 'dhcp'):
                    # set all NIC's to DHCP
                    self.__setAllDHCP()
                else:
                    # set NIC's according kickstart
                    self.__setNIC(model)
        except PKVMError:
            raise
        except Exception as e:
            raise PKVMError("083")
