#======
# Copyright (c) IBM Corp. 2011, 2015.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======


#
# IMPORTS
#
from subprocess import Popen, PIPE
import logging
import os
import re
import shutil
import subprocess
import traceback

from controller.pkvmerror import PKVMError
from modules.scriptbase import ScriptBase


#
# CONSTANTS
#
DEFAULT_TZ = "America/New_York"
ZONEINFO_FILE = "/usr/share/zoneinfo/zone.tab"
ZONEINFO_DIR = "/usr/share/zoneinfo/"


#
# CODE
#
class Timezone(ScriptBase):
    """
    Handles timezone functionalities
    """
    def __init__(self):
        """
        Constructor
        """
        self.__logger = logging.getLogger(__name__)
        self.__tz = None
        self.__mountDir = None
        self.__entries = self.__loadTimezones()

    def __loadTimezones(self):
        """
        Loads all the timezones available

        Returns:
            [str]: list of timezones
        """
        try:
            fd = open(ZONEINFO_FILE)
            content = fd.readlines()
            fd.close()

            result = []

            for line in content:
                if line.startswith("#"):
                    continue

                parts = line.strip().split()

                if len(parts) < 3:
                    continue

                result.append(parts[2])

            result.sort()

            return result
        except Exception as e:
            raise PKVMError("029")

    def getEntries(self):
        """
        Returns all the timezones available

        Returns:
            [str]: list of timezones available
        """
        return self.__entries

    def setTimezone(self, tz):
        """
        Sets timezone

        Args:
            tz (str): timezone
        """
        if tz in self.__entries:
            self.__tz = tz
        else:
            self.__tz = DEFAULT_TZ

    def writeConfig(self, mountDir = ''):
        """
        Write timezone configuration
        """
        try:
            tzFile = os.path.join(ZONEINFO_DIR, self.__tz)
            localTimeFile = "/etc/localtime"
            if mountDir:
                tzFile = mountDir + tzFile
                localTimeFile = mountDir + localTimeFile

            shutil.copy(tzFile, localTimeFile)
        except Exception as e:
            self.__logger.critical("Failed to write timezone configuration")
            self.__logger.critical("%s" % (e))

    def setUTC(self, flag, mountDir = ''):
        """
        Write configuration file to tells UTC is being used or not

        Args:
            flag (bool): True to use UTC, False otherwise
        """
        try:

            adjtimeFile = "/etc/adjtime"
            if mountDir:
                adjtimeFile = mountDir + adjtimeFile

            fd = open(adjtimeFile)
            content = fd.read()
            fd.close()

            newContent = content

            if flag and not "UTC" in content:
                if "LOCAL" in content:
                    newContent = re.sub("LOCAL", "UTC", content)
                else:
                    newContent += "UTC\n"
            elif not "LOCAL" in content:
                if "UTC" in content:
                    newContent = re.sub("UTC", "LOCAL", content)
                else:
                    newContent += "LOCAL\n"

            fd = open(adjtimeFile, "w")
            fd.write(newContent)
            fd.close()
        except Exception as e:
            raise PKVMError("030")

    def setDateTime(self, datetime):
        """
        Set date time of the system

        Args:
            datetime (dict): Date and time.
                Valid keys: year, day, month, hour, minute, second
        """

        year = datetime["year"]
        month = datetime["month"]
        day = datetime["day"]
        hour = datetime["hour"]
        min = datetime["minute"]
        sec = datetime["second"]

        # command to restore SELinux configurations
        cmd = "hwclock --set --date='%s/%s/%s %s:%s:%s' > /dev/null" % (month, day, year, hour,
                min, sec)

        # SELinux restoration failed: error
        if subprocess.call(cmd, shell=True) != 0:
            self.__logger.critical("Error to set datetime using hwclock")

            raise PKVMError("036")
        # Set system clock the same as hardware clock
        subprocess.call(['hwclock', '-s'], shell = False)

    def onPostInstall(self, model):
        """
        Handles the post install event

        Args:
            model (Model): installation data
        """
        try:
            tz = model.get('tz')
            isUTC = model.get('isUTC')
            self.__mountDir = model.get('mountDir')
            if tz:
                self.setTimezone(tz)
                self.writeConfig(self.__mountDir)
                self.setUTC(isUTC, self.__mountDir)
            else:
                self.__logger.critical("Failed to set timezone (%s)" % tz)
        except PKVMError:
            raise
        except Exception as e:
            raise PKVMError("082")
